@@ -2,17 +2,19 @@ package Catmandu::Fix::code;
 
 use Catmandu::Sane;
 
-our $VERSION = '0.9505';
+our $VERSION = '1.07';
 
-use Catmandu::Util qw(is_code_ref);
+use Catmandu::Util qw(check_code_ref);
 use Moo;
 
+with 'Catmandu::Fix::Inlineable';
+
 has code => (
-    is => 'ro', 
-    default => sub { return sub { } },
-    isa => sub {
-        die "code must be a CODE reference" unless is_code_ref($_[0])
-    }
+    is      => 'ro',
+    default => sub {
+        return sub { }
+    },
+    isa => \&check_code_ref,
 );
 
 around BUILDARGS => sub {
